% Prototype LRMC generation model - simplified national model

% Author : Brian Kirtlan June 2013
% Import data based on GEM input sheet data structure)
% Calculates LRMCs and build a system LRMC based on historical generation build timing
% Alternate LRMC calculated using different input assumptions (scenarios)
% Scenario 1 = Corporate (higher discount rate, tax),
% Scenario 2 = Govt Dept (lower discount rate , no tax),
% Scenario 3 = Corporate but with 1988 asset devaluation,
% Scenario 4 = Govt Dept but with 1988 asset devaluation
% Last updated November 2013

ReImportData = 0; % 1 to reimport generation data from Excel GEM sheet, otherwise uses prebuilt .mat file (need to set to 1 if changing sourcefile)
SourceFile = 'C:\DataFiles\MATLABWORK\GenLRMCModel\GEMinputdata - HistoricalPlant.xls'; % GEM source file for importing into the GenDataFile below
LocationFile = 'C:\DataFiles\MATLABWORK\GenLRMCModel\GEMinputdata - 2 regions and transmission - July 2013.xls'; % GEM transmission data source file for importing into the GenDataFile below
GenDataFile = 'GenDataHist.mat'; % pre built .mat file used to store generation data  
DemandFile = 'C:\DataFiles\MATLABWORK\GenLRMCModel\HistoricalGenRequired_GWh.xlsx'; % this is based on a combination of actual build (for early years) and actual generation plus a margin for reserves for later
StartYear = 1905; % LRMC model start year 
nNPVYears = 50; % Sets a max horizon for LRMC NPV calculations
nYears = 109; % number of model years 
nRuns = 1; % used if applying a random capex factor to produce confidence limits
RandomOn = 0; % 1 = on, 0 = off (CAPEX variation determined by RandomCAPEXFactor below)
RandomCAPEXFactor = 0.1; % adjust CAPEX by +/- this amount (0.1 = +-10% change) 
IncludeCO2inNPV = 0; % include carbon costs in the NPV calculation (when not using the full LRMC setting below)
CAPEXInNZD = 1; % Set depending on whether using GEM input (the CAPEX is in foreign currency) or the HVDC analysis inputs ($NZ)  
UseStandardPlantLives = 1; % use technology based plant-lives specified in the GEM input file for LRMC calculation. If not 1 then the MDS-specific plant exogenous retirement years will be used where specified and plant life will be extended to the current year where the standard life has already been exceeded.  
ApplyAnnualFuelFactors = 1; % used to apply a factor to the variable LRMC to adjust for changes in historical fuel prices
CorpYear = 1988; % year of corporatisation (affects when the scenarios switches such as coporate tax, WACC, and revals kick in)
RevalRatio = 1; %0.75 ratio to apply to pre-corporatisation capital expenditure based on the 1988 ECNZ asset re-valuation

disp(' ');
disp('Starting run');

tic;

MDSId = 'mds5'; % GEM mds to use for setting carbon tax assumptions (leave empty to use the default below)
CTaxDefault = 25; % assume $25/tonne as a default 
WUCInt = 0.101; % Real interest rate to apply to WUC where build time exceeds 1 year (not used)
LRMCDiscountRate = 0.101; % 0.045 post-tax real rate of return required on assets (switch to using pre-tax if tax = 0) 
GovtLRMCDiscountRate = 0.101; %real rate of return required on assets (used in Govt Dept scenarios)
TaxRate = 0; %0.28 Set to 0 to ignore any tax impacts in the LRMC calculation (application depends on scenario)

WriteOffPlantAtEOL = 0; % write off any residual asset value of plant at end of life in the LRMC calculation (get a tax benefit);

RevisedExRates = [{'nzd'},1.0;
{'usd'},0.6;
{'eur'},0.5;
{'jpy'},55.0];

% Fuel prices ($/GJ)
FuelRates = [{'Gas'},7.15;
    {'Coal'},5.15;
    {'Lig'},2.26;
    {'Dsl'},31.27];

% Fuel price annual adjustments ($/GJ)
HeavyFuelOilPriceReal = [14.27*ones(1,(1974-StartYear)) 14.27 14.35 16.69 16.84 15.04 17.71 25.48 25.16 28.66 26.70 29.45 26.04 20.63 19.61 18.78 14.05 15.10 16.07 14.75 12.08 11.72 12.83 11.57 11.63 10.94 12.53 16.71 14.18 12.69 12.83 12.92 16.68 16.09 15.73 20.10 16.18 19.92 19.66 19.61 19.61]; % $/GJ 2012 Prices (ex GST) 2013 MBIE Prices report 1990 to 2012 (2013 appended, earlier values assume earliest value available).
WholesaleNaturalGasPriceReal = [4.28.*ones(1,(1979-StartYear)) 4.28 5.23 4.60 3.99 3.79 3.97 4.10 4.47 5.34 4.58 4.43 4.28 4.37 4.42 4.44 4.51 4.61 4.72 4.84 4.95 4.38 3.93 3.88 4.03 4.43 5.18 4.92 5.86 6.44 5.96 7.56 7.79 7.22 7.15 7.15]; % $/GJ 2012 Prices (ex GST) 2013 MBIE Prices report 1990 to 2012 (2013 appended, earlier values based on commercial price movements and assume earliest value available prior to that).
WholesaleDieselPriceReal = [22.59.*ones(1,(1974-StartYear)) 22.59 22.72 26.43 26.67 23.82 28.04 40.35 39.84 45.39 42.04 46.32 40.39 33.79 34.45 33.36 23.38 25.14 19.78 19.61 18.68 17.40 17.42 17.80 17.95 16.33 17.27 23.37 22.01 19.82 20.24 23.24 27.11 30.90 28.42 36.23 25.67 27.57 31.41 31.27 31.27]; % $/GJ 2012 Prices (ex GST) 2013 MBIE Prices report 1990 to 2012 (2013 appended, earlier values assume earliest value available).
WholesaleCoalPriceReal = [4.03.*ones(1,1989-StartYear) 4.03 3.82 3.68 3.50 3.42 3.30 3.01 3.03 2.98 2.92 2.93 2.89 2.84 2.89 3.02 3.36 3.67 3.87 4.28 4.51 4.79 4.87 4.88 5.06 5.15]; % $/GJ Statistics NZ coal price index converted to real prices - see Coal_price_Estimation.xlxs.

% reset random number generator so that using a fixed seed (this is the old MATLAB approach to random numbers) - comment out to use a random seed 
rand('twister',123);  %#ok<RAND>

% load or update data
if ReImportData == 1
    disp(['Updating input data using ' SourceFile]);
    
    [~,PlantMapTxt] = xlsread(SourceFile,'Generation','PltMap');
    [~,PlantsTxt] = xlsread(SourceFile,'Generation','Plants');
    PlantID = PlantsTxt(:,1);
    PlantName = PlantsTxt(:,2);
    [TechData,TechDataTxt] = xlsread(SourceFile,'Technology and fuel','TechData');
    TechTxtWidth = size(TechDataTxt,2) - size(TechData,2); 
    TechHeaders = TechDataTxt(1,TechTxtWidth+1:end);
    TechType = TechDataTxt(2:end,1);
    TechFuel = TechDataTxt(2:end,2);
    TechFuelGroup = TechDataTxt(2:end,3);
    
    [GenData,GenDataTxt] = xlsread(SourceFile, 'Generation', 'GenData');
    GenTxtWidth = size(GenDataTxt,2) - size(GenData,2); 
    GenHeaders = GenDataTxt(1,GenTxtWidth+1:end);
    TechLoc = 2;
    SubLoc = 3;
    GenTech = GenDataTxt(2:end,TechLoc);
    GenSub = GenDataTxt(2:end,SubLoc);
    
    [Retirements,RetirementsTxt] = xlsread(SourceFile,'Generation','Retirements');
    FixedRetirement = Retirements(1:end,1);

    [~,LocationsTxt] = xlsread(LocationFile, 'Locations', 'Locations');
    [TxDataVal,TxDataTxt] = xlsread(LocationFile, 'Transmission', 'TxData');
     
    [EmitVal,EmitTxt] = xlsread(SourceFile,'Emissions','Emit');
    [CTaxVal,CTaxTxt] = xlsread(SourceFile,'Emissions','ctax');
    
    Inflation = xlsread(SourceFile,'Macro','Inflation');
    [FxRates, ~] = xlsread(SourceFile,'Macro','Exrate');
    
    [~,Locations] = xlsread(LocationFile,'Locations','Locations');
    
    [DemandFCVal, DemandFCTxt] = xlsread(DemandFile,'Sheet1','A1:B110'); % demand data not actually used now as not dynamically building generation stack)
    
    % lookup columns as liable to change
    MWLoc = strmatch('MW',GenHeaders,'exact');
    GWhLoc = strmatch('GWh',GenHeaders,'exact');
    vOMLoc = strmatch('vOM',GenHeaders,'exact');
    fOMLoc = strmatch('fOM',GenHeaders,'exact');
    FDCLoc = strmatch('FDC',GenHeaders,'exact');
    CCostLoc = strmatch('CCost',GenHeaders,'exact');
    CurrLoc = strmatch('Curr',GenHeaders,'exact');
    ConCostLoc = strmatch('ConCost',GenHeaders,'exact');
    EarliestYrLoc = strmatch('ErlyYr',GenHeaders,'exact');
    HRLoc = strmatch('HR',GenHeaders,'exact');
    FOFLoc = strmatch('FOF',GenHeaders,'exact');
    
    MW = GenData(:,MWLoc); % installed MW
    GWh = GenData(:,GWhLoc); % max GWh
    vOM  = GenData(:,vOMLoc);% variable O&M $/MWh
    fOM  = GenData(:,fOMLoc); % fixed O&M $/KW
    FDC  = GenData(:,FDCLoc); % fuel delivery cost ($/GJ)
    CCost  = GenData(:,CCostLoc); % capital cost - expressed in O/S currency ($NZ if Curr = 1) Curr/KW
    Curr  = GenData(:,CurrLoc); % currency ID : 1 = NZD (NZ Dollar), 2 = USD (US dollar), 3 = EUR (euro), 4 = JPY (Japanese Yen)
    ConCost  = GenData(:,ConCostLoc); % capital connection cost ($NZ)
    EarliestYr  = GenData(:,EarliestYrLoc); % earliest construction year
    HR = GenData(:,HRLoc); % heat rate (GJ/GWh)
    FOF = GenData(:,FOFLoc); % outage factor (i.e 1 - assumed utilisation)
    
    ConCost(isnan(ConCost)) = 0; % this version of MATLAB seems to be importing empties as NaNs instead of 0s but only in some cases
    vOM(isnan(vOM)) = 0;

    PltLifeLoc = strmatch('PltLife',TechHeaders,'exact');
    TechPltLife = TechData(:,PltLifeLoc);
    DepRateLoc = strmatch('DepRate',TechHeaders,'exact');
    TechDepRate = TechData(:,DepRateLoc);
    CapFacLoc = strmatch('CapFacTech',TechHeaders,'exact');
    TechCapFac = TechData(:,CapFacLoc);
    FXExposeLoc = strmatch('Exposure',TechHeaders,'exact');
    TechFXExpose = TechData(:,FXExposeLoc);
    
    save(GenDataFile,'GenData','PlantID','PlantName','MW','GWh','vOM','fOM','FDC','CCost','Curr','ConCost','EarliestYr',...
    'HR','FOF','GenTech','GenSub','TechType','TechFuel','TechFuelGroup','TechPltLife','TechDepRate','TechCapFac',...
    'TechFXExpose','EmitVal','EmitTxt','CTaxVal','CTaxTxt','FxRates','DemandFCVal','DemandFCTxt','Locations','Inflation','FixedRetirement');
         
else
    disp(['Loading input data ' GenDataFile]);
    load(GenDataFile)    
end;

% separate out new plant and exclude any specific plant 

IncludeList = ones(length(PlantID),1);

% Cull out any 0MW plant is avoid NaNs (shouldn't be any - those expansions providing mainly GWh have a notional MW number as well)
IncludeList(MW==0) = 0;

% convert empty Excel cells converted to NaNs, to 0s
fOM(isnan(fOM)) = 0;
FDC(isnan(FDC)) = 0;

NewPlantIdx = logical(~isnan(EarliestYr).*IncludeList);

PlantID = PlantID(NewPlantIdx);
PlantName = PlantName(NewPlantIdx);
MW = MW(NewPlantIdx);
GWh = GWh(NewPlantIdx);
vOM = vOM(NewPlantIdx);
fOM = fOM(NewPlantIdx);
FDC = FDC(NewPlantIdx);
CCost = CCost(NewPlantIdx);
Curr = Curr(NewPlantIdx);
ConCost = ConCost(NewPlantIdx);
EarliestYr = EarliestYr(NewPlantIdx);
HR = HR(NewPlantIdx);
FOF = FOF(NewPlantIdx);
GenTech = GenTech(NewPlantIdx);
GenSub = GenSub(NewPlantIdx);

ResultStore = zeros(nRuns);

nGen = length(PlantID);
nTech = length(TechType);
RandomCapexAdj = zeros(nGen,1);

nScenarios = 4;
ScenarioNames = [{'Corporate'} {'Govt Dept'} {'Govt Dept - Asset Reval'} {'Default'}];

% pre-allocate cost variables
PlantType = zeros(nGen,1); % type ref #
PlantLife = zeros(nGen,1); % years
FuelPrice = zeros(nGen,1); % $/GJ
DepRate = zeros(nGen,1); % percent expressed as a proportion
CapFactor = zeros(nGen,1); % proportion
FXExpose = zeros(nGen,1); % proportion
FuelEmit = zeros(nGen,1); % tonnes / PJ
LRMC = zeros(nGen,1); % $/MWh
FixedLRMC = zeros(nGen,1); % $/MWh
VarLRMC = zeros(nGen,1); % $/MWh
ScenarioLRMC = zeros(nGen,nScenarios); % $/MWh
FixedScenarioLRMC = zeros(nGen,nScenarios);% $/MWh
VarScenarioLRMC = zeros(nGen,nScenarios);% $/MWh
HVDC_LRMC_Comp = zeros(nGen,1); % $/MWh
LRMC_SystemCost_Comp = zeros(nGen,1); % $/MWh
        
% and region variables
GenRegion = cell(nGen,1);
GenIsland = cell(nGen,1);

% Get required total GWh generation (demand data not actually used now as not dynamically building generation stack)
ImportFCStartYear = DemandFCVal(1,1);
mStart = StartYear;  % default start time specified in the model settings above
mEnd = StartYear + nYears - 1;
if ImportFCStartYear > StartYear
    disp('Warning - the model start year begins earler than demand data is specified for');     
    mStart = ImportFCStartYear;
end;    
if StartYear + nYears - 1 > DemandFCVal(end,1)
    disp('Warning - the length of the modelled period exceeds the demand data provided');
    mEnd = DemandFCVal(end,1);
end;
NatDemand = DemandFCVal(mStart-ImportFCStartYear+1:mEnd-ImportFCStartYear+1,2);
ModelYears = DemandFCVal(mStart-ImportFCStartYear+1:mEnd-ImportFCStartYear+1,1);
if mEnd - mStart + 1 < nYears
    nYears = mEnd - mStart + 1;
    disp(['Warning - modelling period shortened to ' num2str(nYears) ' years due to insufficient demand data. Check demand input file.']);
end;

% fetch lifetime, depreciation, capacity factors, FX exposure, region/island, fuel price and price location factors
WindTypeLoc = strmatch('Wind',TechType,'exact');
for i = 1:nGen
    TypeLoc = strmatch(GenTech(i),TechType,'exact');
    
    if isempty(TypeLoc)
        disp(['Warning - couldn''t find a tech type for ''' PlantName{i} '''. Using default ''IL''']);
        TypeLoc = strmatch('IL',TechType,'exact');
    end;
    PlantType(i) = TypeLoc; 
    % if not using standard plant life, life based on larger of actual life or standard life  (assuming fixed build date = earliest date)   
    if UseStandardPlantLives == 1
        PlantLife(i) = TechPltLife(TypeLoc);
    else    
        if FixedRetirement(i) > 0 % only applicable for import of historical plant with fixed build/retirement dates 
            PlantLife(i) = FixedRetirement(i)-EarliestYr(i)+1;
        else
            if mEnd - EarliestYr(i) + 1 <= TechPltLife(TypeLoc)
                PlantLife(i) = TechPltLife(TypeLoc);
            else
                PlantLife(i) = mEnd - EarliestYr(i) + 1;
            end;
        end;
    end;    
    DepRate(i) = TechDepRate(TypeLoc);
    CapFactor(i) = TechCapFac(TypeLoc);
    FXExpose(i) = TechFXExpose(TypeLoc);
    EmitLoc = strmatch(TechFuel(TypeLoc),EmitTxt,'exact');
    if ~isempty(EmitLoc)
        FuelEmit(i) = EmitVal(EmitLoc);
    end;
    SubLoc = strmatch(GenSub(i),Locations(:,1),'exact');
    if isempty(SubLoc)
        error(['The generation list has a substation (' GenSub{i} ') that is not included in the ''Locations'' list']);
    else
        GenRegion{i} = Locations{SubLoc,3};
        GenIsland{i} = Locations{SubLoc,2};
    end;
    FuelLoc = strmatch(TechFuel(TypeLoc),FuelRates(:,1),'exact');
    if isempty(FuelLoc)
        FuelPrice(i) = 0;
    else
        FuelPrice(i) = FuelRates{FuelLoc,2};
    end;
end;

% rebuild the GWh figures based on MW and utilisation assumptions
for i = 1:nGen
    GWh(i) =  CapFactor(i) * MW(i) * 365 * 24 / 1000;
end;

% carbon tax
CarbonTax = zeros(1,nYears); % $/tonne 
MDSLoc = strmatch(MDSId,CTaxTxt,'exact');
if ~isempty(MDSLoc)
    CTaxDate = CTaxVal(:,1);
    CTaxRates = CTaxVal(:,MDSLoc);
    for i = 1:length(CTaxDate)    
        YearLoc = max(CTaxDate(i)-StartYear + 1,1);
        CarbonTax(YearLoc:end) = CTaxRates(i);
    end;
else
    CarbonTax(:) = CTaxDefault;
end;

Deflator = zeros(1,nYears);
LRMCDiscountFactor = zeros(1,nYears);
GovtLRMCDiscountFactor = zeros(1,nYears);
Deflator(1) = 1;
LRMCDiscountFactor(1) = 1;
GovtLRMCDiscountFactor(1) = 1;
for i = 2:nYears
    Deflator(i) = Deflator(i-1)/(1+Inflation(min(i,length(Inflation)),2));
    LRMCDiscountFactor(i) = 1/(1+LRMCDiscountRate)^(i-1);
    GovtLRMCDiscountFactor(i) = 1/(1+GovtLRMCDiscountRate)^(i-1);
end;

Year = StartYear:StartYear+nYears-1;
NPVYearBuiltCost = zeros(nRuns,nYears);
NPVAccumBuiltCost = zeros(nRuns,nYears);

% Carbon Tax Adjustments ($/GJ)
nEmit = length(EmitVal);
EmitImpactDollarsPerGJ = zeros(nEmit,nYears);
for i = 1: nYears
    EmitImpactDollarsPerGJ(:,i) = CarbonTax(i).*EmitVal/1000000; % $/GJ
end;

HeavyFuelOilPriceReal =  HeavyFuelOilPriceReal + EmitImpactDollarsPerGJ(4,:);
WholesaleNaturalGasPriceReal = WholesaleNaturalGasPriceReal + EmitImpactDollarsPerGJ(3,:);
WholesaleDieselPriceReal = WholesaleDieselPriceReal + EmitImpactDollarsPerGJ(4,:);
WholesaleCoalPriceReal = WholesaleCoalPriceReal + EmitImpactDollarsPerGJ(1,:);

% Set up variable LRMC scale factors to adjust for historical movements in fuel costs and carbon charges
AnnualFuelFactor = ones(nTech,nYears);
if ApplyAnnualFuelFactors == 1
    for i = 1:nTech
        if strmatch(TechFuel(i),'Gas')
            AnnualFuelFactor(i,:) = WholesaleNaturalGasPriceReal./WholesaleNaturalGasPriceReal(end);
        elseif strmatch(TechFuel(i),'Dsl')
            AnnualFuelFactor(i,:) = WholesaleDieselPriceReal./WholesaleDieselPriceReal(end);
        elseif strmatch(TechFuel(i),'FuelOil')
            AnnualFuelFactor(i,:) = HeavyFuelOilPriceReal./HeavyFuelOilPriceReal(end);
        elseif strmatch(TechFuel(i),'Coal')
            AnnualFuelFactor(i,:) = WholesaleCoalPriceReal./WholesaleCoalPriceReal(end);
        end;
    end;
end;

disp(['Running LRMC model from ' num2str(ModelYears(1)) ' to ' num2str(ModelYears(end))'.'])

for Run = 1:nRuns
    
    % randomise capital costs
    for i = 1:nGen
        RandomCapexAdj(i) = rand*(RandomCAPEXFactor*2) - RandomCAPEXFactor;
    end;

    % build time set to 1 year by default
    BuildTime  = ones(nGen,1)*1;

    CheckNPV = zeros(nGen,1); % $m
    CheckLRMCComp = zeros(nGen,1); % $m
    SystemCost = zeros(nGen,1); % $m

    GWhOutput = zeros(nGen,nYears); % GWh (used for LRMC calc)
    RequiredGWhOutput = zeros(nGen,nYears); % GWh (used for build and NPV)
    FuelUse = zeros(nGen,nYears); % GJ
    CO2Emit = zeros(nGen,nYears); % tonnes
    CO2Cost = zeros(nGen,nYears); % $m
    CAPEX  = zeros(nGen,nYears); % $m
    OpenBookValue = zeros(nGen,nYears); % $m (asset opening book value - nominal)
    CloseBookValue = zeros(nGen,nYears); % $m (asset closing book value - nominal)
    Depreciation = zeros(nGen,nYears); % $m (real)
    VariableOM = zeros(nGen,nYears); % $m
    FixedOM = zeros(nGen,nYears); % $m
    FuelDelivery = zeros(nGen,nYears); % $m
    FuelCost = zeros(nGen,nYears); % $m
    CashIn = zeros(nGen,nYears); % $m
    CashOut = zeros(nGen,nYears); % $m
    FixedCashOut = zeros(nGen,nYears); % $m
    VarCashOut = zeros(nGen,nYears); % $m
    SystemCashOut = zeros(nGen,nYears); % $m
    CheckRevenue = zeros(nGen,nYears); % $m
    TotalSystemCost = zeros(nGen,nNPVYears); % $m

    % accumulate LRMC values
    
    % Scenario 1 = Corporate (higher discount rate, tax), 
    % Scenario 2 = Govt Dept (lower discount rate , no tax), 
    % Scenario 3 = Corporate but with 1988 asset devaluation, 
    % Scenario 4 = Govt Dept but with 1988 asset devaluation    
    
    for scn = 1:nScenarios 
        if scn == 1 % Corporate
            ScnTaxRate = TaxRate;
            ScnDF = LRMCDiscountFactor;
            ApplyCapexRatio = 1;
        elseif scn == 2 % Govt Dept
            ScnTaxRate = 0;
            ScnDF = GovtLRMCDiscountFactor;
            ApplyCapexRatio = 0;
        elseif scn == 3 % Corporate with 1988 deval 
            ScnTaxRate = TaxRate;
            ScnDF = LRMCDiscountFactor;
            ApplyCapexRatio = 1;
        elseif scn == 4 % Govt Dept with 1988 deval
            ScnTaxRate = 0;
            ScnDF = GovtLRMCDiscountFactor;
            ApplyCapexRatio = 1;
        else
            disp(['Need to specify input assumptions for scenario ' num2str(scn)]);
        end;

        for i = 1:nGen

            StartOp = BuildTime(i) + 1;
            EndOp = BuildTime(i) + PlantLife(i);

            GWhOutput(i,StartOp:min(nYears,EndOp)) = GWh(i);

            % Capex

            if CAPEXInNZD == 1
                USDRate = FxRates(2,1);
                EURRate = FxRates(3,1);
                YenRate = FxRates(4,1);
                if Curr(i) == 2
                    DifferenceInFXRate = USDRate/RevisedExRates{2,2} - 1;
                elseif Curr(i) == 3
                    DifferenceInFXRate = EURRate/RevisedExRates{3,2} - 1;
                elseif Curr(i) == 4
                    DifferenceInFXRate = YenRate/RevisedExRates{4,2} - 1;
                else                    
                    DifferenceInFXRate = 0;
                end;
                AnnBuildCapex = (CCost(i) * MW(i) / 1000 * (1 + (FXExpose(i) * DifferenceInFXRate))) * (1+(RandomCapexAdj(i)*RandomOn)) / BuildTime(i);
            else
                AnnBuildCapex = (CCost(i) * MW(i) / 1000 * (1/FxRates(FxRates(:,2)==Curr(i),1))) * (1+(RandomCapexAdj(i)*RandomOn)) / BuildTime(i);   
            end;            
            if ApplyCapexRatio == 1 && (EarliestYr(i) < CorpYear) % uses the actual build date in EarliestYr to decide whether to reval down
                CAPEX(i,1:BuildTime(i)) = AnnBuildCapex * RevalRatio;
            else
                CAPEX(i,1:BuildTime(i)) = AnnBuildCapex;
            end;
            CAPEX(i,BuildTime(i)) = CAPEX(i,BuildTime(i)) + ConCost(i); % add connection costs into the last build year (assumes this is also rebuilt along with plant at end of life)
            % Accumulate build CAPEX into asset book
            for j = 1:BuildTime(i)      
                CloseBookValue(i,j) = (CAPEX(i,j)./Deflator(j))+(OpenBookValue(i,j)*(1 + WUCInt)); % inflate subsequent year CAPEX for book value, and include interest on WUC (i.e existing book value)
                OpenBookValue(i,j+1) = CloseBookValue(i,j);
            end;
            % and start to depreciate once built
            for j = BuildTime(i)+1:EndOp %nYears
                DepreciationNom = OpenBookValue(i,j) * DepRate(i); % diminishing value depreciation for tax calculation     
                CloseBookValue(i,j) = max(OpenBookValue(i,j) - DepreciationNom,0);
                Depreciation(i,j) = (OpenBookValue(i,j)- CloseBookValue(i,j)) * Deflator(j); % Deflate to real figures
                OpenBookValue(i,j+1) = CloseBookValue(i,j);
            end;
            if WriteOffPlantAtEOL == 1
                Depreciation(i,EndOp) = Depreciation(i,EndOp) + (Deflator(EndOp) * CloseBookValue(i,EndOp)); % Deflate to a real figure
                CloseBookValue(i,EndOp) = 0;
                OpenBookValue(i,EndOp+1) = 0;
            else
                OpenBookValue(i,EndOp+1:end) = CloseBookValue(i,EndOp);
                CloseBookValue(i,EndOp+1:end) = CloseBookValue(i,EndOp);
            end;
            % Operation and maintenance
            VariableOM(i,:) = GWhOutput(i,:) .* vOM(i) / 1000; % $m = GWh * $/MWh / 1000
            FixedOM(i,StartOp:EndOp) = MW(i) * fOM(i) / 1000; % $m = MW * $/KW / 1000  
            % Carbon and fuel costs (Carbon tax is applied as a ratio to annual total variable costs for the plant rather than being included in the LRMC)
            FuelUse(i,:) = GWhOutput(i,:) .* HR(i); % GJ =  GWh * GJ/GWh
            %CO2Emit(i,:) = FuelUse(i,:) .* FuelEmit(i) / 1000000; % tonnes = GJ * tonnes/PJ / 10^6
            %CO2Cost(i,:) = CO2Emit(i,:) .* CarbonTax / 1000000; % $m = tonnes * $/tonnes / m   
            % Fuel cost (variable LRMC scaled later based on actual build timing)
            FuelCost(i,:) = FuelUse(i,:) .* FuelPrice(i) / 1000000; % $m = GJ * $/GJ / m
            % Fuel delivery
            FuelDelivery(i,:) = FuelUse(i,:) .* FDC(i) / 1000000; % $m = GJ * $/GJ / m
            % Cashflows
            CashOut(i,:) = CAPEX(i,:) + VariableOM(i,:) + FixedOM(i,:) + CO2Cost(i,:) + FuelCost(i,:) + FuelDelivery(i,:); 
            FixedCashOut(i,:) = CAPEX(i,:) + FixedOM(i,:); % splitting into fixed and variable LRMC components
            VarCashOut(i,:) = VariableOM(i,:) + CO2Cost(i,:) + FuelCost(i,:) + FuelDelivery(i,:); % splitting into fixed and variable LRMC components
            CashIn(i,:) = (CashOut(i,:) - CAPEX(i,:) + Depreciation(i,:)) * ScnTaxRate; % Cash in is just the tax benefit from the project (associated with op expenses and depreciation)    
            % LRMC
            LRMC(i) = sum((CashOut(i,:) - CashIn(i,:)) .* ScnDF) ./ sum(GWhOutput(i,:).*ScnDF) * 1000  / (1-ScnTaxRate); 
            % Fixed LRMC components
            FixedLRMC(i) = sum((FixedCashOut(i,:) - CashIn(i,:)) .* ScnDF) ./ sum(GWhOutput(i,:).*ScnDF) * 1000  / (1-ScnTaxRate);
            % Variable LRMC components
            VarLRMC(i) = sum(VarCashOut(i,:) .* ScnDF) ./ sum(GWhOutput(i,:).*ScnDF) * 1000  / (1-ScnTaxRate);

            % System Cost per GWh (and total LRMC based NPV costs)
            if IncludeCO2inNPV == 1;
                SystemCashOut(i,:) = CAPEX(i,:) + VariableOM(i,:) + FixedOM(i,:) + FuelCost(i,:) + FuelDelivery(i,:) + CO2Cost(i,:);
            else
                SystemCashOut(i,:) = CAPEX(i,:) + VariableOM(i,:) + FixedOM(i,:) + FuelCost(i,:) + FuelDelivery(i,:);
            end;
            LRMC_SystemCost_Comp(i) = sum((SystemCashOut(i,:)) .* ScnDF) ./ sum(GWhOutput(i,:).*ScnDF) * 1000; 

            % check
            CheckRevenue(i,:) = LRMC(i) .* GWhOutput(i,:) / 1000;
            CheckNPV(i) = sum(((CheckRevenue(i,:) - CashOut(i,:)) - (CheckRevenue(i,:) - (CashOut(i,:) - CAPEX(i,:) + Depreciation(i,:))) * ScnTaxRate) .* ScnDF);% cashflows: revenue in, less op and capex out, less cash tax out
            CheckLRMCComp(i) = sum(LRMC(i)-VarLRMC(i)-FixedLRMC(i));

        end;

        ScenarioLRMC(:,scn) = LRMC; 
        FixedScenarioLRMC(:,scn) = FixedLRMC;
        VarScenarioLRMC(:,scn) = VarLRMC;

        NonHVDCLRMC = LRMC-(HVDC_LRMC_Comp);

        % check LRMC totals ok (should be 0 or very close to)
        CheckNPV(isnan(CheckNPV))=0;
        CheckTot = sum(CheckNPV);
        if abs(CheckTot) > 0.001
            disp(['WARNING - the LRMC calculation in the script is broken: Scenario ' num2str(scn)])
        end;
        CheckLRMCFixedVar = sum(CheckLRMCComp);
        if abs(CheckLRMCFixedVar) > 0.001
            disp(['WARNING - the LRMC split between fixed and variable costs in the script is broken: Scenario ' num2str(scn)]);
        end;

    end;

    % calculate a system LRMC based on the actual build dates (imported as the earliest build date)
    
    BasePlantYearLRMC = zeros(nGen,nYears); % LRMC based on the existing corporate structure
    ChargedPlantYearLRMC = zeros(nGen,nYears); % Attempt to quantify an amount that would be been recovered by the generator owner/structure at the time
    PlantYearGWh = zeros(nGen,nYears);

    % Corporate base scenario = 1
    CorpScnRef = 1;
    % plant that isn't already retired still has LRMC applied to output past their 'life' 
    %(extending the life in the input data has only a small impact on LRMC due to discounting) 
    % builds a base comparison LRMC using the current corporate sturuture and LRMC calculation
    % and a 'charged' LRMC that calculates LRMCs based on organisational structure pre/post corporatisation (with corporatisation asset reval)
    for i = 1:nGen
        PlantStart = EarliestYr(i)-mStart+1;       
        if FixedRetirement(i) == 0
            PlantEnd = nYears;
        else
            PlantEnd = min(nYears,FixedRetirement(i)-mStart+1);
        end;
        for j = PlantStart:PlantEnd
            BasePlantYearLRMC(i,j) = FixedScenarioLRMC(i,CorpScnRef) + AnnualFuelFactor(PlantType(i),j).* VarScenarioLRMC(i,CorpScnRef);  
            if j < CorpYear-mStart+1 % (pre-post corporatisation)
                ChargedPlantYearLRMC(i,j) = (FixedScenarioLRMC(i,4) + AnnualFuelFactor(PlantType(i),j).* VarScenarioLRMC(i,4));  
            else
                ChargedPlantYearLRMC(i,j) = (FixedScenarioLRMC(i,3) + AnnualFuelFactor(PlantType(i),j).* VarScenarioLRMC(i,3)); % assuming charging using the revalued asset values 
            end;
            PlantYearGWh(i,j) = GWh(i);
        end;
    end;
    PlantYearGWh(PlantYearGWh<0) = 0; % clean up the retirement 'negative GWhs' in the input data
    BasePlantYearCost = BasePlantYearLRMC .* PlantYearGWh / 1000; % $/MWh * GWh / 1000 = $ million 
    ChargedPlantYearCost = ChargedPlantYearLRMC .* PlantYearGWh / 1000; % $/MWh * GWh / 1000 = $ million
    YearGWh = sum(PlantYearGWh,1); % GWh
    BaseYearCost = sum(BasePlantYearCost,1); % $ million
    ChargedYearCost = sum(ChargedPlantYearCost,1); % $ million   
    BaseYearSystemLRMC = BaseYearCost./YearGWh * 1000; % $ million / Gwh * 1000 = $/MWh
    ChargedYearSystemLRMC = ChargedYearCost./YearGWh * 1000; % $ million / Gwh * 1000 = $/MWh    
    BasePlantCost = sum(BasePlantYearCost,2);
    ChargePlantCost = sum(ChargedPlantYearCost,2);

    [HistBuildDates, HistBuildIdx] = sort(EarliestYr);
    HistPlantOrder = PlantName(HistBuildIdx);
    HistLRMC = ScenarioLRMC(HistBuildIdx,CorpScnRef);
    HistPlantGWh = GWh(HistBuildIdx);

    % dump a cut of the total cost array for pasting into the costing spreadsheet 
    DumpStart = 1974;
    SS_BaseLRMCDump = BaseYearCost(DumpStart-mStart+1:end);
    SS_ChargeLRMCDump = ChargedYearCost(DumpStart-mStart+1:end);

    figure(3); plot(ModelYears,[BaseYearSystemLRMC; ChargedYearSystemLRMC]); 
    hold on; 
    title('Av. per unit cost $2013');
    axis([mStart mEnd 0 150]);
    hold off;
   
end;

disp(['Done in ' num2str(toc)]);